#include "SusiePictureWrapper.h"
namespace HongliangSoft {
  namespace Utilities {
    SusiePictureWrapper::SusiePictureWrapper(HMODULE module) : SusieWrapper(module) {
      getPictureHandler = (GetPictureHandler)GetProcAddress(module, "GetPicture");
      getPictureInfoHandler = (GetPictureInfoHandler)GetProcAddress(module, "GetPictureInfo");
      getPreviewHandler = (GetPictureHandler)GetProcAddress(module, "GetPreview");
    }
    void SusiePictureWrapper::Dispose(bool disposing) {
      if (!this->Disposed && disposing) {
        this->getPictureInfoHandler = NULL;
        this->getPictureHandler = NULL;
        this->getPreviewHandler = NULL;
      }
      SusieWrapper::Dispose(disposing);
    }

    Bitmap* SusiePictureWrapper::GetPictureFromFile(String* file) {
      return this->GetPictureFromFile(file, SusieWrapper::noProcess);
    }
    Bitmap* SusiePictureWrapper::GetPictureFromFile(String* file, ProgressCallback* callback) {
      if (!(this->CanGetPicture))
        throw new NotSupportedException();
      return this->GetBitmap(file, this->getPictureHandler, callback);
    }
    Bitmap* SusiePictureWrapper::GetPictureFromStream(Stream* stream) {
      return this->GetPictureFromStream(stream, SusieWrapper::noProcess);
    }
    Bitmap* SusiePictureWrapper::GetPictureFromStream(Stream* stream, ProgressCallback* callback) {
      return this->GetBitmap(stream, this->getPictureHandler, callback);
    }
    Byte SusiePictureWrapper::GetBytes(Stream* stream) __gc[] {
      if (stream == NULL)
        throw new ArgumentNullException("stream");
      if (!stream->CanRead)
        throw new ArgumentException("stream", "ǂݍ߂ȂXg[łB");
      Byte memory[];
      if (stream->GetType() == __typeof(MemoryStream))
        memory = static_cast<MemoryStream*>(stream)->ToArray();
      if (stream->CanSeek) {
        memory = new Byte[stream->Length];
        stream->Read(memory, 0, memory->Length);
      }
      else {
        const bufferSize = 4096;
        Byte buf[] = new Byte[bufferSize];
        int readed = stream->Read(buf, 0, bufferSize);
        if (readed < bufferSize)
          memory = buf;
        else {
          MemoryStream* ms = new MemoryStream();
          ms->Write(buf, 0, readed);
          while ((readed = stream->Read(buf, 0, bufferSize)) > 0)
            ms->Write(buf, 0, readed);
          memory = ms->ToArray();
        }
      }
      return memory;
    }
    Bitmap* SusiePictureWrapper::GetPreviewFromFile(String* file) {
      return this->GetPictureFromFile(file, SusieWrapper::noProcess);
    }
    Bitmap* SusiePictureWrapper::GetPreviewFromFile(String* file, ProgressCallback* callback) {
      if (!(this->CanGetPreview))
        throw new NotSupportedException();
      return this->GetBitmap(file, this->getPreviewHandler, callback);
    }
    Bitmap* SusiePictureWrapper::GetPreviewFromStream(Stream* stream) {
      return this->GetPreviewFromStream(stream, SusieWrapper::noProcess);
    }
    Bitmap* SusiePictureWrapper::GetPreviewFromStream(Stream* stream, ProgressCallback* callback) {
      if (!(this->CanGetPreview))
        throw new NotSupportedException();
      return this->GetBitmap(stream, this->getPreviewHandler, callback);
    }
    Bitmap* SusiePictureWrapper::GetBitmap(String* file, GetPictureHandler handler, ProgressCallback* callback) {
      IntPtr ptr = Marshal::StringToCoTaskMemAnsi(file);
      try {
        return this->GetBitmap(ptr.ToPointer(), 0, FROM_FILE, handler, callback);
      }
      __finally {
        Marshal::FreeCoTaskMem(ptr);
      }
    }
    Bitmap* SusiePictureWrapper::GetBitmap(Stream* stream, GetPictureHandler handler, ProgressCallback* callback) {
      Byte memory[] = SusiePictureWrapper::GetBytes(stream);
      GCHandle handle = GCHandle::Alloc(memory, GCHandleType::Pinned);
      try {
        return this->GetBitmap(handle.AddrOfPinnedObject().ToPointer(), memory->Length, FROM_MEMORY, handler, callback);
      }
      __finally {
        handle.Free();
      }
    }
    Bitmap* SusiePictureWrapper::GetBitmap(void* ptr, int size, int flags, GetPictureHandler handler, ProgressCallback* callback) {
      HLOCAL info, data;
      if (handler(ptr, size, flags, &info, &data, callback, 0) == 0) {
        try {
          BITMAPINFO* bmpinfo = (BITMAPINFO*)LocalLock(info);
          BITMAPINFOHEADER bmpheader = bmpinfo->bmiHeader;
          Bitmap* bmp = new Bitmap(bmpheader.biWidth, bmpheader.biHeight);
          BitmapData* bmpdata = bmp->LockBits(System::Drawing::Rectangle(0, 0, bmp->Width, bmp->Height),
                                              ImageLockMode::WriteOnly, PixelFormat::Format24bppRgb);
          int size = bmpdata->Width * 3;
          int stride = bmpdata->Stride;
          char* dataptr = (char*)LocalLock(data);
          char* scan0 = (char*)bmpdata->Scan0.ToPointer() + (stride * (bmpdata->Height - 1));
          for (int i = 0; i < bmpdata->Height; i++) {
            CopyMemory(scan0, dataptr, size);
            scan0 -= stride;
            dataptr += stride;
          }
          bmp->UnlockBits(bmpdata);
          return bmp;
        }
        __finally {
          LocalUnlock(info);
          LocalFree(info);
          LocalUnlock(data);
          LocalFree(data);
        }
      }
      else
        return NULL;
    }
    PictureInfo* SusiePictureWrapper::GetPictureInfoFromFile(String* filePath) {
      if (!(this->CanGetPictureInfo))
        throw new NotSupportedException();
      IntPtr ptr = Marshal::StringToCoTaskMemAnsi(filePath);
      try {
        PICTINFO pic;
        this->getPictureInfoHandler(ptr.ToPointer(), 0, 0, &pic);
        String* text = NULL;
        if (pic.info != NULL) {
          text = new String((char*)LocalLock(pic.info));
          LocalUnlock(pic.info);
          LocalFree(pic.info);
        }
        return new PictureInfo(pic.left, pic.top, pic.width, pic.height, pic.hRes, pic.vRes, pic.colorDepth, text);
      }
      __finally {
        Marshal::FreeCoTaskMem(ptr);
      }
      return NULL;
    }
    PictureInfo* SusiePictureWrapper::GetPictureInfoFromFileImage(IntPtr image, long size) {
      throw new NotSupportedException();
      return NULL;
    }
  }
}