#include "SusieWrapper.h"

namespace HongliangSoft {
  namespace Utilities {
    void SusieWrapper::ThrowWin32Error() {
      char buffer[256];
      DWORD error = GetLastError();
      FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM, NULL, error, 0, buffer, 256, NULL);
      String *message = new String(buffer);
      throw new Win32Exception(error, message);
    }
    SusieWrapper::~SusieWrapper() {
      this->Dispose(false);
    }
    void SusieWrapper::Dispose() {
      this->Dispose(true);
      GC::SuppressFinalize(this);
    }
    void SusieWrapper::Dispose(bool disposing) {
      if (!this->disposed) {
        disposed = true;
        FreeLibrary(this->module);
        if (disposing) {
          this->module = NULL;
          this->isSupportedHandler = NULL;
          this->getPluginInfoHandler = NULL;
          this->configurationDlgHandler = NULL;
        }
      }
    }
  }
}
#include "SusiePictureWrapper.h"
#include "SusieArchiveWrapper.h"
namespace HongliangSoft {
  namespace Utilities {
    SusieWrapper* SusieWrapper::LoadSpi(String* path) {
      IntPtr ptr = Marshal::StringToCoTaskMemAnsi(path);
      HMODULE module = LoadLibrary((LPSTR)(void*)ptr);
      Marshal::FreeCoTaskMem(ptr);
      if (!module)
        ThrowWin32Error();
      try {
        GetPluginInfoHandler handler = (GetPluginInfoHandler)GetProcAddress(module, "GetPluginInfo");
        const int size = 8;
        char buffer[size];
        int result = handler(0, buffer, size);
        if (result == 0)
          throw new Exception(S"vOC̏擾ł܂B");
        if (buffer[2] == 'I')
          return new SusiePictureWrapper(module);
        else if (buffer[2] == 'A')
          return new SusieArchiveWrapper(module);
        throw new NotSupportedException(String::Concat(S"T|[g̃vOC`łF", new String(buffer)));
      } catch (Exception* ex) {
        FreeLibrary(module);
        throw ex;
      }
    }
    SusieWrapper::SusieWrapper(HMODULE module) {
      isSupportedHandler = (IsSupportedHandler)GetProcAddress(module, "IsSupported");
      getPluginInfoHandler = (GetPluginInfoHandler)GetProcAddress(module, "GetPluginInfo");
      configurationDlgHandler = (ConfigurationDlgHandler)GetProcAddress(module, "ConfigurationDlg");
      String* plugin = this->GetPluginInfo(0);
    }

    bool SusieWrapper::IsSupported(String *filepath) {
      FileStream* fs = NULL;
      try {
        fs = new FileStream(filepath, FileMode::Open);
        return this->IsSupported(filepath, fs->Handle);
      }
      __finally {
        if (fs != NULL)
          fs->Close();
      }
    }
    bool SusieWrapper::IsSupported(String *filepath, IntPtr fileHandle) {
      IntPtr ptr = Marshal::StringToCoTaskMemAnsi(filepath);
      try {
        return Convert::ToBoolean(isSupportedHandler((LPSTR)ptr.ToPointer(), fileHandle.ToInt32()));
      }
      __finally {
        Marshal::FreeCoTaskMem(ptr);
      }
    }
    SusiePluginInfo* SusieWrapper::GetPluginInfo() {
      if (!(this->CanGetPluginInfo))
        return NULL;
      ArrayList* list = new ArrayList();
      String* apiVersion = GetPluginInfo(0);
      String* about = GetPluginInfo(1);
      int index = 2;
      String* ext;
      while ((ext = this->GetPluginInfo(index++)) != NULL) {
        list->Add(__box(FileType(ext, this->GetPluginInfo(index++))));
      }
      return new SusiePluginInfo(apiVersion, about, list);
    }
    String* SusieWrapper::GetPluginInfo(int index) {
      const int size = 128;
      char buffer[size];
      int result = this->getPluginInfoHandler(index, buffer, size);
      return (result == 0) ? NULL : new String(buffer);
    }
    void SusieWrapper::ShowConfiguration(IntPtr parent) {
      if (!(this->CanConfigurationDlg))
        throw new NotSupportedException();
      this->configurationDlgHandler((HWND)parent.ToPointer(), 1);
    }
    void SusieWrapper::ShowAbout(IntPtr parent) {
      if (!(this->CanConfigurationDlg))
        throw new NotSupportedException();
      this->configurationDlgHandler((HWND)parent.ToPointer(), 0);
    }
    int SusieWrapper::NoProcess(int count, int countMax, long data) {
      return 0;
    }
  }
}
